/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2009 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf;

import java.util.Map;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.bf.proxies.StubComponentFactory;
import org.objectweb.fractal.util.Fractal;

/**
 * An abstract class to help plugin developers in building new plugins. It
 * provides a mock implementation of the
 * {@link BindingFactoryPlugin#createStub(Component, String, BindHints)} method.
 * Implementors can provide implementations through the hook points exposed.
 * 
 * @param <E>
 *            the generic type of the {@link ExportHints} of this plugin
 * @param <B>
 *            the generic type of the {@link BindHints} of this plugin
 */
@Deprecated
public abstract class AbstractBindingFactoryPlugin<E extends ExportHints, B extends BindHints>
		implements BindingFactoryPlugin<E, B> {

	Logger log = java.util.logging.Logger
			.getLogger(AbstractBindingFactoryPlugin.class.getCanonicalName());

	private BindingFactoryClassLoader cl;

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getBindingFactoryClassLoader()
	 */
	public BindingFactoryClassLoader getBindingFactoryClassLoader() {
		return this.cl;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#setBindingFactoryClassLoader(BindingFactoryClassLoader)
	 */
	public void setBindingFactoryClassLoader(BindingFactoryClassLoader cl) {
		this.cl = cl;

	}

	/**
	 * Do nothing. Sub classes may customize this behaviour.
	 * 
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#finalizeStub(org.objectweb.fractal.api.Component,
	 *      java.util.Map)
	 */
	public void finalizeStub(Component stub,
			Map<String, Object> finalizationHints)
			throws BindingFactoryException {
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#createStub(org.objectweb.fractal.api.Component,
	 *      java.lang.String, java.util.Map)
	 */
	public Component createStub(Component client, String itfname,
			BindHints bindHints) throws StubGenerationException {

		if (client == null) {
			throw new IllegalArgumentException(
					"The component owning the client interface to bind can't be null");
		} else if (itfname == null || itfname.equalsIgnoreCase("")) {
			throw new IllegalArgumentException(
					"Can't find client interface with name:" + itfname);
		} else if (bindHints == null) {
			throw new IllegalArgumentException(
					"You didn't specify a map of hints to create the stub.");
		}
		String componentName = null;
		try {
			componentName = Fractal.getNameController(client).getFcName();
		} catch (NoSuchInterfaceException e1) {
			componentName = client.toString();
		}
		log.info("Creating stub for component " + componentName + ",interface:"
				+ itfname + ",bindHints:" + bindHints.toString());

		Component stub;
		InterfaceType interfaceType;
		try {
			ComponentType itf = (ComponentType) (client).getFcType();

			interfaceType = itf.getFcInterfaceType(itfname);

			StubComponentFactory stubComponentFactory = getStubComponentFactory();
			log.finest("StubComponentFactory: "
					+ stubComponentFactory.toString());

			stub = stubComponentFactory.createStubComponent(interfaceType,
					bindHints);

		} catch (Exception e) {
			throw new StubGenerationException(
					"Some error occurred while creating a stub", e);
		}

		return stub;
	}

	/**
	 * Abstract implementation, to be overriden by subclasses.
	 * 
	 * @return
	 */
	protected abstract StubComponentFactory getStubComponentFactory();

	/**
	 * Do nothing. Sub classes may customize this behaviour.
	 * 
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#finalizeSkeleton(org.objectweb.fractal.api.Component,
	 *      java.util.Map)
	 */
	public void finalizeSkeleton(Component skel,
			Map<String, String> finalizationHints)
			throws BindingFactoryException {

	}

}
