/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2009 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.proxies;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;

import org.objectweb.fractal.bf.BindHints;

/**
 * A StubContentGenerator implementation using JDK dynamic proxies as content
 * part of stub components.
 * 
 * @see StubContentGenerator
 */
@Deprecated
public abstract class AbstractDynamicProxyStubContentGenerator<B extends BindHints>
		implements StubContentGenerator<B> {

	public Object getStubComponentContentInstance(Class<?> clazz, B bhints)
			throws GeneratorException {
		checkClass(clazz);

		InvocationHandler ih = createInvocationHandler(bhints, clazz);

		return createAndInstantiateStubContent(clazz, ih);
	}

	/**
	 * @param params
	 * @param clazz
	 * @return
	 */
	protected abstract InvocationHandler createInvocationHandler(B bindHints,
			Class<?> clazz);

	/**
	 * Return the array of interfaces the proxy will implement
	 * 
	 * @param clazz
	 *            the functional interface
	 * @return
	 */
	protected Class<?>[] proxyInterfaces(Class<?> clazz) {
		return new Class[] { clazz };
	}

	/**
	 * @param clazz
	 * @throws GeneratorException
	 */
	private void checkClass(Class<?> clazz) throws GeneratorException {
		if (!clazz.isInterface()) {
			throw new GeneratorException(
					"Impossible to generate the content of a stub component using JDK dynamic proxies for a class that it's not an interface (was: "
							+ clazz.getCanonicalName() + ")");
		}
	}

	/**
	 * @param
	 * @param ih
	 *            the invocation handler for the dynamic proxy to istantiate
	 * @return
	 */
	private Object createAndInstantiateStubContent(Class<?> clazz,
			InvocationHandler ih) {
		Object proxyComponentContentInstance = Proxy.newProxyInstance(clazz
				.getClassLoader(), proxyInterfaces(clazz), ih);
		return proxyComponentContentInstance;
	}

}
